#encoding: utf8

from math import sqrt, pi
import random

from .rack import ModeRR, ModePOP
from .cow_up import CowUp

class Rack:
    """Enum of modes"""
    REDREBEL, POP= range(2)



class Game:
    """Game class, core of the game
     """

    def __init__(self, modes, difficulty, (width, height)): 
        """Game constructor"""
        rack_classes = {Rack.REDREBEL: ModeRR, Rack.POP: ModePOP}
        self.difficulty = difficulty
        self.health_capacity = 10000*(8./difficulty)
        self.health_current = self.health_capacity
        self.last_generation_count = 0
        self.list_of_bullets = []
        self.list_of_cows_up = []
        self.list_of_cows_side = []
        self.players = [rack_classes[mode](difficulty, mode, self) for mode in modes]
        self.width = width
        self.height = height
        self.number_cows_up_gone = 0
        self.number_cows_up_created = 0
        self.number_cows_up_shot = 0
        self.number_cows_up_missed = 0
        self.number_cows_up_created_all = 0
        self.bool_print_once = False
        
    def reactualize_life(self, bullet):
        """Reactualize life"""
        self.health_current -= bullet.damage

    def collide_on_limit_bullets(self, bullet):
        """For detect bullet which escaping from the screen"""
        if bullet.y > self.height + 250:
            #self.health_current -= bullet.damage
            self.list_of_bullets.remove(bullet)

    def collide_on_limit_cows_up(self, cow_up):
        """For detect bullet which escaping from the screen"""
        if cow_up.y < -125:
            self.list_of_cows_up.remove(cow_up)
            self.number_cows_up_gone = self.number_cows_up_gone + 1
            
    def collide_bullets_and_cows_up(self, bullet, cow_up):
    	"""Bullet and CowUp collision"""
        hb_half_length = cow_up.hit_box_length / 2. + bullet.radius
        if cow_up.x - hb_half_length < bullet.x < cow_up.x + hb_half_length and cow_up.y - hb_half_length < bullet.y < cow_up.y + hb_half_length and bullet.y < self.height:
            #self.score += (2 * bullet.damage * (500 + bullet.number_type**7))/100
            if bullet in self.list_of_bullets:
            	self.list_of_bullets.remove(bullet)
            
            self.list_of_cows_up.remove(cow_up)
            self.number_cows_up_gone = self.number_cows_up_gone + 1
            self.number_cows_up_shot = self.number_cows_up_shot + 1
            
    def continuum_generate_cows(self, area_radius):
        """Create cows up dynamicaly"""
        #if self.number_cows_up_created < 100:
	density_random_factor = 0 #random.randint(0, 3)
	kr = density_random_factor
	global_width = self.width * 16.
	#number_little_cows = random.randint(0 * kr, 1 + kr)
	number_little_cows = random.randint(0, 600)
	if number_little_cows > 565 - self.number_cows_up_missed:
		#for k in xrange(number_little_cows):
		self.list_of_cows_up.append(CowUp(random.uniform(20, self.width - 30), self.height + 30, 0))
		self.number_cows_up_created = self.number_cows_up_created + 1
        
                
    def update(self, keystates):
        """Main game loop component.
	"""
        for player, keystate in zip(self.players, keystates):
            player.update(keystate)
        for bullets in self.list_of_bullets:
            bullets.update()
        for cows_up in self.list_of_cows_up:
            cows_up.update()
        if self.number_cows_up_created < max(100, 100 + self.number_cows_up_missed):
        	self.continuum_generate_cows(600)
        	self.bool_print_once = False
	else:
		if self.bool_print_once == False and len(self.list_of_cows_up) == 0:
			self.number_cows_up_created_all = self.number_cows_up_created_all + self.number_cows_up_created
			self.number_cows_up_missed = self.number_cows_up_missed + self.number_cows_up_created_all - self.number_cows_up_shot
			p = repr(self.number_cows_up_shot) + '/' + repr(self.number_cows_up_created_all)
			self.number_cows_up_created = 0
			print(p)
			self.bool_print_once = True
        for player in self.players:
            for bullets in self.list_of_bullets:
                player.on_collide(bullets)
        for bullet in self.list_of_bullets:
            self.collide_on_limit_bullets(bullet)
        for cow_up in self.list_of_cows_up:
            self.collide_on_limit_cows_up(cow_up)
        for bullet in self.list_of_bullets:
            for cow_up in self.list_of_cows_up:   
            	self.collide_bullets_and_cows_up(bullet, cow_up)
        #print self.number_cows_up_gone
        if self.number_cows_up_gone > 100 :
        	#print 'k'
        	return False
	else:
		return True
        

        
           




           
