from math import sqrt, pi, cos, sin

from .keys import (LEFT, RIGHT, SHOOT)
from .bullet import Bullet

class Rack :
    """Rack class definition defined by :
    _ its mode (a number)
    _ its game (a Game object)
    _ its position x (x, in pixels)
    _ its position y (y, in pixels)
    _ its magnet jauge recovery (in percents)
    _ its magnet jauge speed recovery (in percents)
    _ its magnet efficient area (in pixels)
    _ its hitbox length (in pixels)
    _ its speed (in rad by move)
    _ its circle location radius
    _ its score
    """

    def __init__(self,  difficulty, mode, game):
        """Rack constructor"""
        self.game = game
        self.mode = mode
        self.score = 0
        self.hit_box_length = 10
        self.health_current = 10000
        self.x = 300
        self.y = 40
        self.half_square_length = 600
        self.cooldown = 30
        self.bool_cooldown = False


    def on_collide(self, bullet):
        """Bullet and Rack collision"""
        hb_half_length = self.hit_box_length / 2. + bullet.radius
        if self.x - hb_half_length < bullet.x < self.x + hb_half_length and self.y - hb_half_length < bullet.y < self.y + hb_half_length:
            #self.score += (2 * bullet.damage * (500 + bullet.number_type**7))/100
            self.game.reactualize_life(bullet)
            self.game.list_of_bullets.remove(bullet)


    def circle_of_admission(self, list_of_bullets):
        """Rack magnet"""
        if self.magnet_jauge_recovery == 100.:
            self.magnet_jauge_recovery = 0.
            for bullet in list_of_bullets:
                if sqrt((self.x - bullet.x)**2 + (self.y - bullet.y)**2) < self.magnet_efficient_area + bullet.radius:
                    self.score += (2 * bullet.damage * (500 + bullet.number_type**7))/1000
                    self.game.list_of_bullets.remove(bullet)



class ModeRR(Rack) :
    """ModeRouge sub-class definition self-attributes:
    _ its half square length

    """

    def __init__(self, difficulty, mode, game):
        """ModeRouge constructor"""
        Rack.__init__(self, difficulty, mode, game)
        # Define speed
        frames_needed = (8+difficulty/4.)*60
        perimeter = self.half_square_length*8
        self.speed = 1.75 * perimeter / frames_needed



    def update(self, keystate):
        """ModeRedRebel movement"""
        if LEFT in keystate:
			self.x = max(self.x - self.speed, 45)#-self.half_square_length + 300)
        if RIGHT in keystate:
			self.x = min(self.x + self.speed, self.half_square_length - 45)
	if SHOOT in keystate:
		if self.cooldown == False and (len(self.game.list_of_bullets) < 2):
			#print len(self.game.list_of_bullets)
			self.game.list_of_bullets.append(Bullet(0, 0, 0, 0, self.x, self.half_square_length, 0))
			self.cooldown = 0
        		self.bool_cooldown = True
        		
	if self.bool_cooldown == True:
		self.cooldown = self.cooldown + 1
	if self.cooldown >= 15:
		self.cooldown = 0
		self.bool_cooldown = False
      	        
        
        
class ModePOP(Rack) :
    """ModePOP sub-class definition self-attributes:
    _ its half square length

    """

    def __init__(self, difficulty, mode, game):
        """ModePOP constructor"""
        Rack.__init__(self, difficulty, mode, game)
        self.x = 2
        self.y = 0
        
    def update(self, keystate):
        """ModeRedRebel movement"""
        if LEFT in keystate:
            self.x = max(self.x - self.speed, -self.half_square_length)
        if RIGHT in keystate:
            self.x = min(self.x + self.speed, self.half_square_length)

