import math

import pyglet
from pyglet.gl import *

from coweverywhere.game.keys import *


class GameRunner(pyglet.window.Window):
    def __init__(self, game, screen_size):
        width, height = screen_size
        pyglet.window.Window.__init__(self, width=width, height=height,
                                            caption='Cows Everywhere !',
                                            resizable=True)
        self.game = game

        self.keystates = [set(), set()]
        #TODO: keymap-independent bindings?
        self.keybindings = {
            # Player 1
            pyglet.window.key.LEFT: (0, LEFT),
            pyglet.window.key.RIGHT: (0, RIGHT),
            pyglet.window.key.SPACE: (0, SHOOT)
        }

	game_textures = pyglet.resource.image('indext.png')
	bullet_textures = pyglet.resource.image('stickballt.png')
	cow_up_textures = pyglet.resource.image('cow_upt.png')
        
        self.rack_textures = [game_textures.get_region(0, 0, 92, 26)]
        self.bullet_textures = [bullet_textures.get_region(0, 0, 43, 50)]
        self.cow_up_textures = [cow_up_textures.get_region(0, 0, 76, 152)]
        
        # Fix anchors
        for img in self.rack_textures:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        
        for img in self.bullet_textures:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        
        for img in self.cow_up_textures:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.

        self.fps_display = pyglet.clock.ClockDisplay()
                                
                             
    def on_key_press(self, symbol, modifiers):
        pyglet.window.Window.on_key_press(self, symbol, modifiers)
        if symbol in self.keybindings:
            player, key = self.keybindings[symbol]
            self.keystates[player].add(key)


    def on_key_release(self, symbol, modifiers):
        #pyglet.window.Window.on_key_release(self, symbol, modifiers)
        if symbol in self.keybindings:
            player, key = self.keybindings[symbol]
            self.keystates[player].discard(key)


    def update(self, dt):
        self.game.update(self.keystates)
        
        
    def on_draw(self):
        self.clear() #TODO

        background_batch = pyglet.graphics.Batch()
        batch = pyglet.graphics.Batch()
        sprites = []

        #TODO: various effects, white hole itself, ...

        # Praw players
        #TODO: circle/square
        for player in self.game.players:
		'''background_sprite = pyglet.sprite.Sprite(self.ship_backgrounds[player.mode],
		                                     x=player.x, y=player.y,
		                                     batch=background_batch)'''
		sprite = pyglet.sprite.Sprite(self.rack_textures[player.mode],
						  x=player.x,
						  y=player.y,
						  batch=batch)
		sprites.append(sprite)
		
	# Draw bullets
        for bullet in self.game.list_of_bullets:
            sprite = pyglet.sprite.Sprite(self.bullet_textures[0], #bullet.number_type],
                                          x=bullet.x,
                                          y=bullet.y,
                                          batch=batch)
            sprite.scale = bullet.radius/10.
            sprites.append(sprite)
            
	# Draw cows up
        for cow_up in self.game.list_of_cows_up:
            sprite = pyglet.sprite.Sprite(self.cow_up_textures[0], #bullet.number_type],
                                          x=cow_up.x,
                                          y=cow_up.y,
                                          batch=batch)
            sprite.scale = cow_up.radius/3.
            sprites.append(sprite)
            
            
        # Draw rack
        background_batch.draw()
        batch.draw()
            
	glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        #self.fps_display.draw()
            
                
